<?php  

require_once(ParticleTraffic_PATH.'lib/db.php');
require_once(ParticleTraffic_PATH.'lib/visit.php');
require_once(ParticleTraffic_PATH.'lib/hit.php');
require_once(ParticleTraffic_PATH.'lib/cached_month.php');
require_once(ParticleTraffic_PATH.'lib/tools.php');
require_once(ParticleTraffic_PATH.'lib/display.php');
//require_once(ParticleTraffic_PATH.'lib/installer.php');
require_once(ParticleTraffic_PATH.'lib/authentication.php');

class ParticleTrafficTracker { 

	protected $dao_loader; 
	protected $db;
	protected $book_id=0; //book_id is your siteIdNumber
	
	public static $visits_table='TonyStatsVisits';
	public static $hits_table='TonyStatsHits';
	public static $cache_table='TonyStatsCachedMonths';
	
	//the amount of time spent idle in seconds before new page views will create a new visit
	static public $visitTimeout=1800;
	
	public static function quickTrack( $options=array() ){		
		$statsTracker = new ParticleTrafficTracker( intval($options['siteIdNumber']) );
		$statsTracker->trackHit( $options ); 
	}
	
	public static function printTrackingJS($siteIdNumber=0){
		echo '<script src="'.ParticleTraffic_URL_DIR.'ParticleTraffic.services.php?mode=tracker_js&siteIdNumber='.intval($siteIdNumber).'" type="text/javascript"></script> '."\n";
	}

	public function __construct($siteIdNumber=0){ 
		$this->db = ParticleTrafficDB::get_instance(); 
		$this->book_id=intval($siteIdNumber);
	} 
	
	//a hit triggered from the browser site
	public function browserSideHit( $options=array() ){
		$visit=$this->getThisVisit();
		$hitVO = new ParticleTrafficHit();	
		$hitVO->set_traffic_visits_id( intval($visit->traffic_visit_id) );
		$url=ParticleTrafficTools::simpleText( $_REQUEST['url'] );
		$hitVO->set_page_url($url); 
		$short_url=(strstr($url,'?')) ? substr($url,0,strpos($url,'?')) : $url;	
		$hitVO->set_page_url_short( $short_url ); 		
		$hitVO->set_page_title( strip_tags( $_REQUEST['t'] ) ); 		
		$hitVO->create();		
	}	
	
	//serverSideHit
	public function trackHit($options=array()){ 
		$visit=$this->getThisVisit(); 
		$hitVO = new ParticleTrafficHit();	
		$hitVO->set_traffic_visits_id(intval($visit->traffic_visit_id));
		$url=$_SERVER['REQUEST_URI'];
		if( strlen($options['url']) ) $url=ParticleTrafficTools::simpleText($options['url']);
		$hitVO->set_page_url($url); 
		$short_url=(strstr($url,'?')) ? substr($url,0,strpos($url,'?')) : $url;	
		$hitVO->set_page_url_short( $short_url ); 		
		$hitVO->set_page_title( strip_tags($options['title']) ); 		
		$hitVO->create();
	}	
	
	public function getVisitId(){
		$session_id=session_id();
		if( !strlen($session_id) )
			throw new Exception('You must start a php session with session_start() for ParticleTraffic to work');
		return intval($this->book_id).'_'.$_SERVER['REMOTE_ADDR'].'_'.$session_id;
	}
	
	public function getThisVisit($withCreate=1,$user_id=-1){ 
		$browserInfo=$this->getBrowserInfo();
		
		if($user_id==-1){
			$u = new User();	
			$user_id = intval($u->uID);
		}
		
		//get by 
		$session_id=$this->getVisitId();
		$visit=ParticleTrafficVisit::retrieveBySession( $session_id );  
		
		$cookies_off_id=intval($this->book_id).'_'.ParticleTrafficTools::simpleText($_SERVER['REMOTE_ADDR']).'_'.ParticleTrafficTools::simpleText($browserInfo['platform']).'_'.ParticleTrafficTools::simpleText($browserInfo['browser']).'_'.$browserInfo['majorVersion'];
		if(!$visit){
			//visit not found - cookies might be disabled so check by just ip now 
			$visit=ParticleTrafficVisit::retrieveByCookiesOffId($cookies_off_id); 
		}
		
		//is this visit still active? 
		if( $visit && !$visit->isVisitActive() ) $visit=NULL;
			
		if($visit){ 
			if(!$visit->users_id && $user_id>0){
				$visit->set_users_id($user_id); 						
				$visit->save();
			}
		}else{
			if(!$withCreate) return false;
			$visit=new ParticleTrafficVisit();	
			$visit->set_session_id($session_id); 
			$visit->set_cookies_off_id($cookies_off_id);
			$visit->set_book_id(intval($this->book_id)); 
			$visit->set_users_id($user_id);
			$visit->set_ip($_SERVER['REMOTE_ADDR']);
			$visit->set_host_name($this->getShortRemoteHostName());		
			if( strlen($_REQUEST['ref']) && $_REQUEST['ref']!=$_REQUEST['url'] && !stristr($_REQUEST['ref'],BASE_URL))	
				$visit->set_referrer_url(str_ireplace('www.','',$_REQUEST['ref']) ); 
			//elseif(!stristr($_SERVER['HTTP_REFERER'],$_SERVER["SERVER_NAME"])) 
			//	$visit->set_referrer_url(str_ireplace('www.','',$_SERVER['HTTP_REFERER']) ); 
			if($_REQUEST['ref'] && $_REQUEST['ref']!=$_REQUEST['url'])
				$visit->set_search_terms(trim($this->getSearchTerms())); 
			$visit->set_platform($browserInfo['platform']);
			$visit->set_browser($browserInfo['browser']);
			$visit->set_browser_version($browserInfo['majorVersion']);		
			if($_REQUEST['fv']) $visit->set_flash_version(intval($_REQUEST['fv']));
			if($_REQUEST['wx'] && $_REQUEST['wy']) $visit->set_resolution(intval($_REQUEST['wx']).' x '.intval($_REQUEST['wy']));
			$visit->save();
		}
		return $visit;
	}	
	
	public function getSearchTerms(){
		$url=$_REQUEST['ref']; 
		$refUrlParts=explode('?',$url);
		$refUrlBase=$refUrlParts[0];
		if(stristr($refUrlBase,$_SERVER["SERVER_NAME"])) return ''; 
		if( strpos($url,'?')<=0 ) return ''; 
		$qStr=substr($url,strpos($url,'?')+1); 
		$keyVals=explode('&',$qStr);
		foreach($keyVals as $keyValPair){ 
			if(!strstr($keyValPair,'=')) continue;
			$parts=explode('=',$keyValPair); 
			if(strtolower($parts[0])=='q') return urldecode($parts[1]);
			if(strtolower($parts[0])=='p' && stristr($_REQUEST['ref'],'yahoo')) return urldecode($parts[1]); 
		}
		return '';
	}

	public function getShortRemoteHostName(){
		$ip=$_SERVER['REMOTE_ADDR'];
		$dashed_ip=str_replace('.','-',$ip);
		$host=gethostbyaddr($ip);
		if($host==$ip) return '';
		return str_replace(array($dashed_ip.'.',$ip.'.'),'',$host);
	}
	
	function getBrowserInfo() {
		// Note: An excellent article on browser IDs can be found at
		// http://www.zytrax.com/tech/web/browser_ids.htm
	
		$SUPERCLASS_NAMES = "gecko,mozilla,mosaic,webkit";
		$SUPERCLASS_REGX  = "(?:".str_replace(",", ")|(?:", $SUPERCLASS_NAMES).")";
	
		$SUBCLASS_NAMES   = "opera,msie,firefox,chrome,safari";
		$SUBCLASS_REGX    = "(?:".str_replace(",", ")|(?:", $SUBCLASS_NAMES).")";
	
		$browser      = "unrecognized";
		$majorVersion = "0";
		$minorVersion = "0";
		$fullVersion  = "0.0";
		$platform     = 'unrecognized';
	
		$userAgent    = strtolower($_SERVER['HTTP_USER_AGENT']);
	
		$found = preg_match("/(?P<browser>".$SUBCLASS_REGX.")(?:\D*)(?P<majorVersion>\d*)(?P<minorVersion>(?:\.\d*)*)/i",$userAgent, $matches);
		if (!$found) {
			$found = preg_match("/(?P<browser>".$SUPERCLASS_REGX.")(?:\D*)(?P<majorVersion>\d*)(?P<minorVersion>(?:\.\d*)*)/i",$userAgent, $matches);
		}
	
		if ($found) {
			$browser      = $matches["browser"];
			$majorVersion = $matches["majorVersion"];
			$minorVersion = $matches["minorVersion"];
			$fullVersion  = $matches["majorVersion"].$matches["minorVersion"];
			if ($browser == "safari") {
				if (preg_match("/version\/(?P<majorVersion>\d*)(?P<minorVersion>(?:\.\d*)*)/i",$userAgent, $matches)){
					$majorVersion = $matches["majorVersion"];
					$minorVersion = $matches["minorVersion"];
					$fullVersion  = $majorVersion.".".$minorVersion;
				}
			}
		}
	
		if (strpos($userAgent, 'linux')) {
			$platform = 'linux';
		}
		else if (strpos($userAgent, 'macintosh') || strpos($userAgent, 'mac platform x')) {
			$platform = 'mac';
		}
		else if (strpos($userAgent, 'windows') || strpos($userAgent, 'win32')) {
			$platform = 'windows';
		}
	
		return array(
			"browser"      => $browser,
			"majorVersion" => $majorVersion,
			"minorVersion" => $minorVersion,
			"fullVersion"  => $fullVersion,
			"platform"     => $platform,
			"userAgent"    => $userAgent);
	}	
	
	/*
	public function saveBrowserInfo(){
		$visit=$this->getThisVisit(0);
		if(!$visit) return false; 
		//$visit->set_platform($_REQUEST['p']);
		//$visit->set_browser($_REQUEST['b']);
		//$visit->set_browser_version($_REQUEST['bv']);
		$visit->set_flash_version(intval($_REQUEST['fv']));
		$visit->set_resolution(intval($_REQUEST['wx']).' x '.intval($_REQUEST['wy']));
		$visit->save();
	}
	*/
	
	protected function cleanup(){
		if( date('i')!=58 ) return;
		$sql='DELETE FROM traffic_hits WHERE created<'.(date('U')-(3600*24*365) ); 
		$this->db->query($sql);
		$sql='DELETE FROM traffic_visits WHERE created<'.(date('U')-(3600*24*365) ); 
		$this->db->query($sql); 		
	}	

} 

?>